/*
 * BackgroundServices.c
 *
 * Created: 04-Apr-17 14:54:25
 *  Author: maticpi
 */ 
#include "BackgroundServices.h"

int score;
int level;
int lives;
char display_msg;	//1,2,3 or 4

void BackgroundServices()
{
	static uint32_t last_time;
	
	if (Has_X_MillisecondsPassed(10,&last_time))
	{
		KBD_Read();
	}
	RefreshLCDandLEDs();
}

void RefreshLCDandLEDs()
{
	switch (display_msg)
	{
		case DISP_MOLE1:
//			LED_ShowMoleNr(1);
			LCD_ShowMoleNr(1);
			break;
		case DISP_MOLE2:
//			LED_ShowMoleNr(2);
			LCD_ShowMoleNr(2);
			break;
		case DISP_MOLE3:
//			LED_ShowMoleNr(3);
			LCD_ShowMoleNr(3);
			break;
		case DISP_MOLE4:
//			LED_ShowMoleNr(4);
			LCD_ShowMoleNr(4);
			break;
		case DISP_MOLE_NONE:
//			LED_ShowMoleNr(0);
			LCD_ShowMoleNr(0);
			break;
		case DISP_MOLE_ALL:
//			LED_ShowMoleNr(5);
			LCD_ShowMoleNr(5);
			break;
		case DISP_REFRESH_SCORE:
			LCD_RefreshScore();
			break;
		case DISP_REFRESH_LEVEL:
			LCD_RefreshLevel();
			break;
		case DISP_REFRESH_LIVES:
			LCD_RefreshLives();
			break;
		case DISP_REFRESH_ALL:
			LCD_RefreshScore();
			LCD_RefreshLevel();
			LCD_RefreshLives();
			break;
		case DISP_START_SCREEN:
			DrawStartScreen();
			break;
		case DISP_GAME_PLAY_SCREEN:
			DrawGamePlayScreen();
			LCD_RefreshScore();
			LCD_RefreshLevel();
			LCD_RefreshLives();
			break;
		case DISP_END_SCREEN:
			lives = -1;
			LCD_RefreshLives();
			DrawGameOver();
			break;
		case DSIP_END_SCORE:
			DrawGameScore();
			break;
		default:	//0 or unknown - do nothing
		break;
	}
	display_msg=0;
}

/*
void LED_ShowMoleNr(char mole_nr)
{
	if (mole_nr == 0) PORTE=0;
	else if (mole_nr == 5) PORTE=0x0f;
	else if ((mole_nr<5) && (mole_nr>0)) PORTE = 1<<(mole_nr-1);
	//else unknown code, do nothing
}
*/
void LCD_RefreshScore()
{
	char txt[10];
	static int oldLength;
	int i,lnt;
	UG_FontSelect(&RFONT_8X12);
	UG_SetForecolor(C_WHITE);
	sprintf(txt,"%d",score);
	lnt=strlen(txt);
	if (lnt<oldLength)
	{
		for (i=0;i<oldLength-lnt; i++) txt[lnt+i]=' ';
		txt[lnt+i]=0;	//termination char
	}
	UG_PutString(56,2,txt);
	oldLength=lnt;
}

void LCD_RefreshLives()
{
	static int oldLives;
	const int lX[3]={271,286,301};
	int i, clives;
			
/*	if (lives > 9)	//GREEN
	else if (lives > 6)	//BLUE
	else if (lives > 3)	//YELLOW
	else 	//RED
*/
	clives=lives;	//copy number of lives first, then adjust
					//so we don't try to draw more than possible
	if (lives == -1)	//force refresh
	{
		lives=0;
		clives=0;
		oldLives=3;
	}
	else
	{
		if (clives > 3) clives=3;
		if (clives < 0) clives=0;
		if (clives == oldLives) return;
	}
	if (clives > oldLives)	//add lives
	{
		for (i=oldLives; i<clives; i++ )
		{
			DrawIcon_P(lX[i],17,14,11,C_BLACK,C_RED,&lifeIco[0][0]);
		}	
	}
	else	//remove lives
	{
		for (i=clives; i<oldLives; i++ )
		{
			UG_FillFrame(lX[i],17,lX[i]+14,17+11,C_BLACK);
		}
	}
	oldLives=clives;
}

void LCD_RefreshLevel()
{
	char txt[10];
	static int oldLength;
	int i,lnt;
	UG_FontSelect(&RFONT_8X12);
	UG_SetForecolor(C_WHITE);
	sprintf(txt,"%d",level);
	lnt=strlen(txt);
	if (lnt<oldLength)
	{
		for (i=0;i<oldLength-lnt; i++) txt[lnt+i]=' ';
		txt[lnt+i]=0;	//termination char
	}
	UG_PutString(56,15,txt);
	oldLength=lnt;
}

void LCD_ShowMoleNr(char mole_nr)
{
	static char oldMolePositions;	//bits 0-3: 1-mole is drawn, 0-mole isn't drawn
	const int mX[4]={80-15,200-15,280-15,200-15};
	const int mY[4]={120-15,120-15,160-15,200-15};
	int i;
	
	if (mole_nr == 0 || mole_nr == 6)	//delete all moles (6 forces delete regardless of oldMolePositions )
	{
		for (i=0; i<4; i++)
		{
			if ((mole_nr == 6) || (oldMolePositions & (1<<i)))	//if present delete else do nothiing
			{
				UG_FillCircle(mX[i]+15,mY[i]+15,15,C_BLACK);
			}
		}
		oldMolePositions=0;
	}
	else if (mole_nr == 5) //draw all moles
	{
		for (i=0; i<4; i++)
		{
			if (!(oldMolePositions & (1<<i)))	//if !present draw else do nothiing
			{
				DrawSprite_P(mX[i],mY[i],30,31,C_GREEN,&sprite[0][0]);
			}
		}
		oldMolePositions=0x0F;
	}
	else if ((mole_nr<5) && (mole_nr>0))	//delete moles on the wrong positions and draw the mole on the correct position
	{
		for (i=0; i<4; i++)
		{
			if ((!(oldMolePositions & (1<<i))) && ((i+1) == mole_nr))	//if !present && needs to be here, draw 
			{
				DrawSprite_P(mX[i],mY[i],30,31,C_GREEN,&sprite[0][0]);
				oldMolePositions |= (1<<i);
			}
			else if ((oldMolePositions & (1<<i)) && ((i+1) != mole_nr))	//else if present and mustn't be, delete else do nothiing
			{
				UG_FillCircle(mX[i]+15,mY[i]+15,15,C_BLACK);
				oldMolePositions &= ~(1<<i);
			}
		}
	}
}

void DrawStartScreen()
{	//&RFONT_8X12 &RFONT_16X26
	UG_FillFrame(0,0,320,100,C_BLACK);
	UG_FillFrame(0,100,320,220,C_GREEN);
	UG_FillFrame(0,220,320,240,C_BLACK);
	UG_FontSelect(&RFONT_16X26);
	UG_SetForecolor(C_WHITE);
	UG_SetBackcolor(C_BLACK);
	UG_PutString(50,20,"WHACK A  OLE");
	UG_SetForecolor(C_RED);
	UG_PutString(185,25,"M");
	UG_FontSelect(&RFONT_8X12);
	UG_SetForecolor(C_WHITE);
	UG_PutString(5,223,"Press OK to start!");
	
	LCD_ShowMoleNr(6);	//deletes moles regardles if they were there or not
}

void DrawGamePlayScreen()
{	//&RFONT_8X12 &RFONT_16X26
	UG_FillFrame(0,0,320,100,C_BLACK);
	UG_FillFrame(0,80,320,240,C_GREEN);
	UG_FontSelect(&RFONT_8X12);
	UG_SetForecolor(C_WHITE);
	UG_PutString(2,2,"Score:");
	UG_PutString(2,15,"Level:");
	UG_PutString(272,2,"Lives");
	UG_DrawRoundFrame(268,15,318,30,4,C_WHITE);
	
	LCD_ShowMoleNr(6);	//deletes moles regardles if they were there or not
}

void DrawGameOver()
{	//&RFONT_8X12 &RFONT_16X26
	int i,x,y;
	x=GetSysTick_us() % (320-32);	//random
	y=GetSysTick_us() % (160-34);
	for (i=0; i<100; i++)
	{
		DrawSprite_P(x+1,y+81,30,31,C_GREEN,&sprite[0][0]);
		x = (x+GetSysTick_us()) % (320-32);
		y = (y+GetSysTick_us()) % (160-34);
	}
	UG_FontSelect(&RFONT_16X26);
	UG_SetForecolor(C_RED);
	UG_PutString(90,100,"GAME OVER");
}


void DrawGameScore()
{	//&RFONT_8X12 &RFONT_16X26
	char txt[10];
	UG_FontSelect(&RFONT_16X26);
	UG_SetForecolor(C_WHITE);
	UG_PutString(60,130,"Final Score:");
	sprintf(txt,"%d",score);
	UG_PutString(120,160,txt);
}
void DrawIcon_P(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t bgClr, uint16_t fgClr, const uint8_t *data)
{
	uint16_t clr;
	int i,j,idx=0;
	char d=0,bit;
	
	ILI9341_setAddrWindow(x,y,x+w-1,y+h-1);
	for (j=0;j<h;j++)
	{
		for (i=0;i<w;i++)
		{
			bit = i & 0x07;
			if (bit == 0)
			{
				d=pgm_read_byte((uint16_t)data+idx);
				if (w-i>8) idx++;
			}
			clr = d & (0x80>>bit) ? fgClr : bgClr; 
			ILI9341_pushColor(clr);
		}
		idx++;
	}
}

const PROGMEM uint8_t lifeIco[11][2]={
{ 0x18, 0x60},	//0001 1000 0110 0000
{ 0x7c, 0xf8},	//0111 1100 1111 1000
{ 0xff, 0xfc},	//1111 1111 1111 1100
{ 0xff, 0xfc},	//1111 1111 1111 1100
{ 0x7f, 0xf8},	//0111 1111 1111 1000
{ 0x7f, 0xf8},	//0111 1111 1111 1000
{ 0x3f, 0xf0},	//0011 1111 1111 0000
{ 0x1f, 0xe0},	//0001 1111 1110 0000
{ 0x0f, 0xc0},	//0000 1111 1100 0000
{ 0x07, 0x80},	//0000 0111 1000 0000
{ 0x03, 0x00}	//0000 0011 0000 0000
};

void DrawSprite_P(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t bgClr, const uint16_t *data)
{
	uint16_t clr;
	uint16_t size=h*w;
	int i;
	
	ILI9341_setAddrWindow(x,y,x+w,y+h);
	for (i=0;i<size;i++)
	{
		clr = pgm_read_word((uint16_t)data+2*i);
		if (!clr) clr=bgClr;
		ILI9341_pushColor(clr);
	}
}

const PROGMEM uint16_t sprite[30][31]={
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xBDD6, 0x6AE8, 0x8367, 0x7B8C, 0xBDD7, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x7B47, 0xF70F, 0xF730, 0xE6AF, 0x49E5, 0x9CD3, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x8C30, 0xE66F, 0xF777, 0xF798, 0xFF52, 0x7326, 0x39E8, 0xBD95, 0xA4D2, 0x8BED, 0x942F, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x8C0E, 0xF732, 0xFF99, 0xFFDB, 0xFF73, 0x6AC7, 0x31A7, 0x9409, 0x7AC5, 0x7264, 0x59C3, 0x6B0A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xAD13, 0xE68E, 0xFF73, 0xFF74, 0xDE4E, 0x3985, 0x3166, 0x39C7, 0x39A6, 0x6246, 0x9344, 0xDD25, 0x9BC4, 0x7B6A, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x8388, 0xE66B, 0xCDAA, 0x49E5, 0x2945, 0x41A4, 0x3964, 0x2925, 0x3186, 0x4229, 0x4A48, 0x83CA, 0xD60C, 0x7B26, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xAD13, 0x93CC, 0xB554, 0xD699, 0x6A63, 0x8B23, 0x7A83, 0xCC44, 0x7A64, 0xA324, 0xC3E5, 0x92C4, 0x51A3, 0x2945, 0x424A, 0x4A48, 0xACCA, 0x7B06, 0x0000, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x93ED, 0x82C5, 0x82A4, 0x8284, 0x92E5, 0x59E4, 0xC463, 0x7AA3, 0x92E4, 0x92C4, 0x59A2, 0xB384, 0xED63, 0xF5A2, 0xF563, 0xBBE3, 0x51A3, 0x3186, 0x4209, 0x72E6, 0x730A, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x72A7, 0x7A64, 0x82C4, 0x8264, 0x9B04, 0x92C4, 0x7243, 0x7225, 0x4185, 0xA489, 0xEDA7, 0xED23, 0xDCA4, 0xA344, 0x4983, 0x5A04, 0x82E3, 0xCCA2, 0xF5A2, 0x9B63, 0x3144, 0x39E8, 0x3965, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x838B, 0x9B45, 0xA304, 0x9B04, 0x7A84, 0x73AF, 0x9D56, 0x3986, 0x6BF0, 0x6BD1, 0x62CA, 0x72A7, 0x51C5, 0x4165, 0x49C7, 0x49A6, 0x41A6, 0x3123, 0x6A63, 0xE563, 0xBCA5, 0x3164, 0x39E8, 0x6B2C, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x9C4F, 0x8264, 0x82A4, 0x94B3, 0xD73D, 0x7BF0, 0x636E, 0x8D57, 0xAE3A, 0xDF5E, 0xB619, 0x31A8, 0x3166, 0x4229, 0x4209, 0x39E8, 0x39E8, 0x39C7, 0x41A4, 0xDD86, 0xCD06, 0x3964, 0x39E8, 0x8C10, 0x0000},
	{0x0000, 0x0000, 0x832B, 0xAC0E, 0x932A, 0xACF3, 0x0000, 0x0000, 0x7B8D, 0xB67A, 0x9D77, 0x39C7, 0x7C73, 0xCF1D, 0xCEFC, 0x9D97, 0x9D56, 0x4209, 0x3146, 0x41A6, 0x49E7, 0x49E7, 0x49C7, 0x41A7, 0x3965, 0x49A3, 0xE5E7, 0xBC43, 0x3124, 0x8C30, 0x0000},
	{0x0000, 0x7228, 0x9A69, 0x8A49, 0x9AEA, 0xA30B, 0xBDB6, 0x0000, 0x7C32, 0x84B4, 0x4209, 0x3966, 0x9D14, 0xA5D8, 0x9D77, 0x7C73, 0x4AAD, 0x3965, 0x5A48, 0x836C, 0x8B8C, 0x72E9, 0x836B, 0x72CA, 0x5A29, 0x5A28, 0x7282, 0xCC64, 0x51A3, 0xA4F3, 0x6248},
	{0x6248, 0x71E7, 0xB2EB, 0x6185, 0x4944, 0x928A, 0x51A6, 0x5A49, 0x4A29, 0x3145, 0x49C7, 0x6AAB, 0x5A69, 0x7C32, 0x530F, 0x4A8D, 0x4A08, 0x72EA, 0xBCF1, 0x93AC, 0x72E9, 0x5226, 0x7AE9, 0xB48F, 0x7B0A, 0x5A29, 0x3964, 0xABC3, 0x51E6, 0xBDB6, 0x72AA},
	{0x72AA, 0x4103, 0x71C6, 0x6185, 0x28A2, 0x8228, 0x6A28, 0x8B8E, 0x838E, 0x838E, 0x8B8E, 0x838E, 0x836E, 0x72EB, 0x5A69, 0x6269, 0x7B2C, 0xAC8F, 0x836B, 0x732A, 0xC5D2, 0xAD10, 0x5206, 0xC531, 0xC532, 0x5A28, 0x5248, 0x7264, 0x8B8C, 0x0000, 0xA4D2},
	{0xA4D2, 0x4924, 0x6986, 0x71A6, 0x5965, 0x71C7, 0x6228, 0x838E, 0x838E, 0x8B8E, 0x8B8E, 0x836E, 0x836E, 0x836E, 0x836E, 0x836D, 0x7B2D, 0x5A28, 0x7AE9, 0x62A8, 0xC5D2, 0xC5D3, 0x51E6, 0xB4F0, 0xC552, 0x834B, 0x62CA, 0x6AC9, 0xAC2B, 0x0000, 0x0000},
	{0x0000, 0x4185, 0x71A6, 0x8A08, 0x79C7, 0x5985, 0x7B2C, 0x836E, 0x838E, 0x838E, 0x838E, 0x838E, 0x836E, 0x836E, 0x836D, 0x836D, 0x836D, 0x6289, 0xA40D, 0x5A47, 0xC5D3, 0xCE34, 0x838C, 0x93ED, 0xC552, 0xA42E, 0x5207, 0x83CE, 0xA42C, 0x0000, 0x0000},
	{0x0000, 0x5207, 0x49A6, 0x5186, 0x51C6, 0x72EB, 0x836E, 0x836E, 0x836E, 0x838E, 0x836E, 0x836E, 0x836E, 0x836D, 0x836D, 0x836D, 0x834D, 0x7AEA, 0x8B8B, 0x5206, 0x8C0D, 0xC5D3, 0x9C6F, 0x836B, 0xC552, 0xB4D0, 0x51E7, 0x5AAA, 0x83CE, 0x0000, 0x0000},
	{0x0000, 0x62A9, 0x7B2B, 0x9C2E, 0x9C2E, 0x7B0B, 0x834D, 0x836E, 0x836E, 0x836E, 0x836E, 0x836E, 0x836E, 0x836D, 0x836D, 0x834D, 0x6AA9, 0xB48F, 0x6A88, 0x8C0D, 0xDED9, 0x8C0E, 0x6288, 0x7B2A, 0xC552, 0xBCF1, 0x5A49, 0x41A6, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x83CE, 0xAC8F, 0xCD72, 0xCD72, 0xCD72, 0x93CD, 0x72EB, 0x836D, 0x836E, 0x836E, 0x836D, 0x836D, 0x836D, 0x72EB, 0x7B2B, 0xBD11, 0xAC4E, 0x4164, 0xD676, 0xFFFE, 0xFFFE, 0xA512, 0x7B0A, 0xC552, 0xB4D0, 0x628A, 0x5A48, 0x838C, 0x6A88, 0x0000},
	{0x0000, 0x0000, 0x72EA, 0xCD72, 0xAC6F, 0xCD72, 0xCD72, 0xBCF1, 0x8B8C, 0x730B, 0x6ACA, 0x6AAA, 0x72EB, 0x8BAD, 0xB4D0, 0xC552, 0xB46F, 0x6A68, 0x83AC, 0xF7BD, 0xFFFE, 0xFFFE, 0xBD95, 0x834B, 0xC552, 0xA42E, 0x72CB, 0x49A5, 0x938A, 0x8B6A, 0x0000},
	{0x0000, 0x0000, 0x83AE, 0x9C2E, 0xA44E, 0x9BED, 0xBCF1, 0xCD72, 0xCD93, 0xCD93, 0xCD92, 0xCD92, 0xCD93, 0xCD72, 0xBCF1, 0xAC4E, 0x7B0A, 0x51E6, 0xDEB7, 0xFFFF, 0xFFFF, 0xFFFE, 0xA4D2, 0x93CD, 0xC572, 0x8B8C, 0x732C, 0x49A5, 0x8B6A, 0x7AC8, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x6AEB, 0x8B4B, 0x7AE9, 0x936B, 0xAC4E, 0xB46F, 0xB4AF, 0xB4B0, 0xB4B0, 0xB46F, 0xAC4E, 0xAC2E, 0x6A88, 0x6AC8, 0xA4CF, 0x9CAF, 0xF7BD, 0xFFFF, 0xFFFE, 0x7B6D, 0xB4D0, 0xC572, 0x72EA, 0x7B0C, 0x4164, 0x7287, 0x7286, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x8C30, 0x5206, 0x4985, 0x59E6, 0x8B4B, 0xA40E, 0xAC2E, 0xAC2E, 0x9BAC, 0x72A8, 0x49A5, 0x942D, 0xC5F3, 0xEF5B, 0xA4D2, 0xBD93, 0xFFDE, 0xFFDE, 0x5A27, 0xCD72, 0xC572, 0x6269, 0x732C, 0x4144, 0x5A05, 0x838D, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x732B, 0x940D, 0x83AB, 0x5A67, 0x5A67, 0x6287, 0x5226, 0x732A, 0x9C6E, 0xC5B3, 0xF79D, 0xFFFF, 0xFFDF, 0x7B6C, 0xDED9, 0xBDB6, 0x834B, 0xCD72, 0xA46F, 0x6AAA, 0x732C, 0x49E7, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0xA4F0, 0xDE97, 0xA4F0, 0xDE97, 0xCE13, 0x83CC, 0x942D, 0xE6FA, 0xA4D1, 0xFFFF, 0xFFFF, 0xFFFF, 0xD69A, 0x83AC, 0x6268, 0xC552, 0xC531, 0x7B0A, 0x730C, 0x730C, 0x5A49, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x9CB2, 0xF79C, 0xAD32, 0xFFFD, 0xFFDD, 0xB553, 0x9C4E, 0xF79D, 0x946F, 0xFFDE, 0xFFFF, 0xFFFF, 0xEF7D, 0x5207, 0xB4D0, 0xC531, 0xA42E, 0x6269, 0x730C, 0x730C, 0x5A48, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x83CE, 0x51E6, 0x942F, 0xB574, 0xEF5A, 0xFFFE, 0xD6B9, 0x9C4E, 0xE71B, 0xA4D1, 0xF79D, 0xF79D, 0xAD13, 0x6AC9, 0xBCF0, 0xB4D0, 0xA40D, 0x7AEA, 0x730C, 0x730C, 0x730C, 0x5228, 0x0000, 0x0000, 0x0000},
	{0x0000, 0x0000, 0x0000, 0x9450, 0x6288, 0x6A87, 0x8B49, 0x8308, 0x6226, 0xBD94, 0x9C90, 0xFFFD, 0xFFBD, 0x8BCC, 0x6AE9, 0x72E9, 0x7B4B, 0x7B2A, 0xAC8F, 0xC552, 0xA42E, 0x8B4B, 0x9C2E, 0x628A, 0x730C, 0x730C, 0x730C, 0x5228, 0x0000, 0x0000, 0x9471},
	{0x9471, 0x6ACA, 0x6246, 0x8329, 0x7AC8, 0x7AE8, 0x8B49, 0x72A7, 0x4164, 0x49E7, 0x734C, 0x9C90, 0xA4F2, 0x730A, 0xAC6E, 0xC531, 0xC531, 0xBCF1, 0xB48F, 0x7B0A, 0x836B, 0xB4D0, 0x5A48, 0x730C, 0x7B4D, 0x7B2D, 0x730C, 0x5207, 0x0000, 0x0000, 0x5206}
};
